{% extends 'area_access/base.html' %}
{% block title %}Farewell{% endblock %}
{% block default_content %}
	<h1>Scan your badge to log out of the {{ area }}.</h1>
{% endblock %}
{% block action %}
	exit_area(badge_number);
{% endblock %}