{% extends 'staff_charges/staff_charges_base.html' %}
{% block staff_charges_content %}
	<div class="col-sm-12">
		<form action="{% url 'end_staff_area_charge' %}" class="form-horizontal" method="post">
			{% csrf_token %}
			<div class="form-group">
				<input type="submit" class="btn btn-primary" style="white-space:normal" value="I am no longer in the {{ area }} on behalf of this user">
			</div>
		</form>
	</div>
{% endblock %}