# Generated by Django 2.2.10 on 2020-05-17 20:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('NEMO', '0015_area_maximum_capacity'),
    ]

    operations = [
        migrations.AddField(
            model_name='area',
            name='reservation_warning',
            field=models.PositiveIntegerField(blank=True, help_text='The number of simultaneous users (with at least one reservation in this area) allowed before a warning is displayed when creating a reservation.', null=True),
        ),
    ]
