{% if project %}
	<form id="tool_control" style="display: none">
		{% csrf_token %}
		<input type="hidden" name="project_id" value="{{ project.id }}">
	</form>
{% endif %}
<div class="row" style="margin:30px 0">
	{% if back %}
		<div class="col-xs-offset-1 col-xs-5" onclick="tool_reservation('{% url 'kiosk_tool_reservation' tool.id customer.id back %}')">
			<div class="alert alert-info" style="text-align:center; margin-bottom:0">
				<span style="font-size:x-large">Go back to reservation</span>
			</div>
		</div>
	{% else %}
		<div class="col-xs-offset-1 col-xs-5" onclick="load_choices('{{ customer.badge_number }}')">
			<div class="alert alert-info" style="text-align:center; margin-bottom:0">
				<span style="font-size:x-large">Go back</span>
			</div>
		</div>
	{% endif %}
	<div class="col-xs-5" onclick="revert()">
		<div class="alert alert-warning" style="text-align:center; margin-bottom:0">
			<span style="font-size:x-large">I've finished using this kiosk</span>
		</div>
	</div>
</div>

<h2>Error</h2>
<h3>{{ message }}</h3>
<script>revert(20)</script>