{% load custom_tags_and_filters %}
<!-- Version -->
<footer class="page-footer">
	{% app_version as app_version %}
	<div class="container">
		{% if app_version %}v {% app_version %}{% else %}&nbsp;{% endif %} - <a href="https://www.nist.gov/cnst" target="_blank">Developed by CNST, NIST</a>
	</div>
</footer>