{% extends 'staff_charges/staff_charges_base.html' %}
{% load mptt_tags %}
{% block staff_charges_content %}
	<form action="{% url 'end_staff_charge' %}" class="form-horizontal" method="post">
		{% csrf_token %}
		<input type="submit" class="btn btn-primary" value="I have finished working on this project">
	</form>
	{% if areas %}
		<div style="height:30px"></div>
		<form action="{% url 'begin_staff_area_charge' %}" method="post">
			{% csrf_token %}
			<p>I have entered a billable area on behalf of this user:</p>
			{% include 'area_access/login_areas.html' with areas=areas user_accessible_areas=user_accessible_areas %}
			<br>
			<input type="submit" class="btn btn-primary" value="Begin billing for area access">
		</form>
	{% endif %}
{% endblock %}