{% load static %}
{% load custom_tags_and_filters %}

<!DOCTYPE html>
<html>
<head>
	<title>Authorization failed</title>
	{# jQuery #}
	<script type="text/javascript" src="{% static "jquery.js" %}"></script>

	{# Bootstrap #}
	<script type="text/javascript" src="{% static "bootstrap/js/bootstrap.js" %}"></script>
	<link rel="stylesheet" type="text/css" href="{% static "bootstrap/css/bootstrap.css" %}"/>
	<link rel="stylesheet" type="text/css" href="{% static "bootstrap/css/bootstrap-theme.css" %}"/>
</head>
<body>
	{% if authorization_failed %}
		{{ authorization_failed|safe }}
	{% else %}
		<div style="padding: 10px; font-size: 16px">
			You do not have a {{ site_title }} user account or your user account has been deactivated. Please contact the {{ facility_name }} staff for assistance.
		</div>
	{% endif %}
</body>
</html>
