# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-10-15 15:06
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('NEMO', '0004_version_1_3_0'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='taskstatus',
            name='notify_secondary_tool_owner',
        ),
        migrations.RemoveField(
            model_name='tool',
            name='secondary_owner',
        ),
        migrations.AddField(
            model_name='taskstatus',
            name='notify_backup_tool_owners',
            field=models.BooleanField(default=False, help_text='Notify the backup tool owners when a task transitions to this status'),
        ),
        migrations.AddField(
            model_name='tool',
            name='backup_owners',
            field=models.ManyToManyField(blank=True, help_text='Alternate staff members who are responsible for administration of this tool when the primary owner is unavailable.', related_name='backup_for_tools', to=settings.AUTH_USER_MODEL),
        ),
    ]
