# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-08-09 14:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('NEMO', '0006_version_1_8_0'),
    ]

    operations = [
        migrations.AddField(
            model_name='tool',
            name='policy_off_between_times',
            field=models.BooleanField(default=False, help_text='Check this box to disable the following policy rules every day between the given times'),
        ),
        migrations.AddField(
            model_name='tool',
            name='policy_off_end_time',
            field=models.TimeField(blank=True, help_text='The end time when policy rules should NOT be enforced', null=True),
        ),
        migrations.AddField(
            model_name='tool',
            name='policy_off_start_time',
            field=models.TimeField(blank=True, help_text='The start time when policy rules should NOT be enforced', null=True),
        ),
        migrations.AddField(
            model_name='tool',
            name='policy_off_weekend',
            field=models.BooleanField(default=False, help_text='Whether or not policy rules should be enfored on weekends'),
        ),
    ]
